#ifndef PROBLEM_H
#define PROBLEM_H

#include <meshagent.h>
#include <boundaryfunction.h>
#include <problemdescriptorbase.h>
#include <laplace.h>
#include <dirichletdata.h>
#include <domainrighthandside.h>
#include <zerodirichletdata.h>

using namespace Gascoigne;


// class for specifying Dirichlet data
class MyDirichletData : public DirichletData
{
  Gascoigne::LaplaceData data;

public:

  MyDirichletData(const Gascoigne::ParamFile& pf,
                  const Gascoigne::LaplaceData& dat) : DirichletData(pf), data(dat)
  {}

  std::string GetName() const { return "Dirichlet Data"; }

  void operator()(DoubleVector &b, const Vertex2d &v, int col) const
  {
    b[0] = 0.;
  }
};


// Problem 2.1 class for specifying problem's right hand side
class MyRightHandSide : public DomainRightHandSide
{
public:

  // every Application-class needs such a function to clone itself. This is required for
  // parallelization. If the class depends on some persistent data, it must be passed
  // to the constructor. We advice to only pass pointers or references to a data object.
  MyRightHandSide* createNew() const
  {
    return new MyRightHandSide;
  }
  

  int GetNcomp() const { return 1; }
  std::string GetName() const { return "Right Hand Side"; }

  double operator()(int c, const Vertex2d &v) const
  {
    return 1.;
  }
};


// main class for defining the problem to solve
class MyProblemDescriptor : public ProblemDescriptorBase
{

  Gascoigne::LaplaceData data;

public:

   std::string GetName() const {  return "Laplace Problem";}

  void BasicInit(const ParamFile & pf)
  {
    GetParamFile() = pf;
    data.BasicInit(pf);

    // equation to solve
    GetEquationPointer() = new Laplace2d(data);
    // definition for right hand side
    GetRightHandSidePointer() = new MyRightHandSide();
    // definition of dirichlet boundary data
    //Problem 2.2 Change to MyDirichletData
    GetDirichletDataPointer() = new ZeroDirichletData(pf);

    ProblemDescriptorBase::BasicInit(pf);
  }
};




class Disc : public BoundaryFunction<2>
{
public:
  std::string GetName() const
  {
    return "Disc";
  }

  void BasicInit(Vertex2d c, double r)
  {
    _c = c;
    _r = r;
  }

  double operator()(const Vertex2d &c) const
  {
    double r = -_r;
    for (int i = 0; i < 2; i++) {
      double dx = c[i] - _c[i];
      r += dx * dx;
    }
    return r;
  }

private:
  double _r;
  Vertex2d _c;
};

/*----------------------------------------------------------------------------*/

class MyMeshAgent : public MeshAgent
{
public:
  MyMeshAgent() : MeshAgent()
  {
    //Problem 2.5 (Extra)
    //Define a circle and project
    // the lower-right edges to it

  }

private:
  Disc D;
};

#endif /* PROBLEM_H */
